/*	Copyright (C) 2001,2002,2003 Ronnie Maor and Michael Brand
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the copyright holders give permission
 * for additional uses of the text contained in its release of XPARAM.
 *
 * The exception is that, if you link the XPARAM library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the XPARAM library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * If you write modifications of your own for XPARAM, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */


#pragma warning (disable: 4101)

/* $ANTLR 2.7.1: "parser.g" -> "xParamParser.cpp"$ */
#include <../src_include/xParamParser.hpp>
#include "antlr/NoViableAltException.hpp"
#include "antlr/SemanticException.hpp"


#include "../xparam/xp_error.h"

using namespace antlr;
using namespace std;


ANTLR_BEGIN_NAMESPACE(xParam_internal)



xParamParser::xParamParser(antlr::TokenBuffer& tokenBuf, int k)
: antlr::LLkParser(tokenBuf,k)
{
	setTokenNames(_tokenNames);
}

xParamParser::xParamParser(antlr::TokenBuffer& tokenBuf)
: antlr::LLkParser(tokenBuf,1)
{
	setTokenNames(_tokenNames);
}

xParamParser::xParamParser(antlr::TokenStream& lexer, int k)
: antlr::LLkParser(lexer,k)
{
	setTokenNames(_tokenNames);
}

xParamParser::xParamParser(antlr::TokenStream& lexer)
: antlr::LLkParser(lexer,1)
{
	setTokenNames(_tokenNames);
}

xParamParser::xParamParser(const antlr::ParserSharedInputState& state)
: antlr::LLkParser(state,1)
{
	setTokenNames(_tokenNames);
}

void xParamParser::value_set(
	 AssignmentListener& listener 
) {
	
	try {      // for error handling
		{
		for (;;) {
			if ((LA(1)==ID||LA(1)==URL||LA(1)==FLAG)) {
				value_set_member(listener);
			}
			else {
				goto _loop3;
			}
			
		}
		_loop3:;
		}
		{
		switch ( LA(1)) {
		case 4:
		{
			match(4);
			break;
		}
		case antlr::Token::EOF_TYPE:
		{
			match(antlr::Token::EOF_TYPE);
			break;
		}
		default:
		{
			throw antlr::NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	catch (ANTLRException& ex) {
		if (inputState->guessing==0) {

			
					throw Error(ex.what());
				

		} else {
			throw ex;
		}
	}
}

void xParamParser::value_set_member(
	 AssignmentListener& listener 
) {
	antlr::RefToken  url = antlr::nullToken;

	
	Handle<Value> val;
	

	
	switch ( LA(1)) {
	case ID:
	{
		member_assignment(listener);
		break;
	}
	case URL:
	{
		url = LT(1);
		match(URL);
		if ( inputState->guessing==0 ) {

			m_url_read_value_set(url->getText(),listener);

		}
		break;
	}
	case FLAG:
	{
		flag_setting(listener);
		break;
	}
	default:
	{
		throw antlr::NoViableAltException(LT(1), getFilename());
	}
	}
}

Handle<Value>  xParamParser::value() {

	Handle<Value> val;

	antlr::RefToken  id = antlr::nullToken;
	antlr::RefToken  url = antlr::nullToken;
	
	try {      // for error handling
		switch ( LA(1)) {
		case TENTATIVE:
		case CHAR:
		case STRING:
		{
			val=basic_type();
			break;
		}
		case TYPENAME:
		{
			val=compound_value();
			break;
		}
		case ID:
		{
			id = LT(1);
			match(ID);
			if ( inputState->guessing==0 ) {

				val=m_constant_or_tentative_string(id->getText());

			}
			break;
		}
		case 12:
		{
			val=list_value();
			break;
		}
		case URL:
		{
			url = LT(1);
			match(URL);
			if ( inputState->guessing==0 ) {

				val=m_url_read_value(url->getText());

			}
			break;
		}
		default:
		{
			throw antlr::NoViableAltException(LT(1), getFilename());
		}
		}
	}
	catch (NoViableAltException& ex) {
		if (inputState->guessing==0) {

			
					string msg;
					if (LT(1)->getType() == Token::EOF_TYPE)
						msg = "unexpected EOF";
					else
						msg = "unexpected token type";
					throw Error(msg);
				

		} else {
			throw ex;
		}
	}
	catch (ANTLRException& ex) {
		if (inputState->guessing==0) {

			
					throw Error(ex.what());
				

		} else {
			throw ex;
		}
	}
	return val;
}

Handle<Value>  xParamParser::basic_type() {

	Handle<Value> val;

	antlr::RefToken  t = antlr::nullToken;
	antlr::RefToken  c = antlr::nullToken;
	antlr::RefToken  s = antlr::nullToken;
	
	switch ( LA(1)) {
	case TENTATIVE:
	{
		t = LT(1);
		match(TENTATIVE);
		if ( inputState->guessing==0 ) {

			val=m_create_tentative_value(t->getText());

		}
		break;
	}
	case CHAR:
	{
		c = LT(1);
		match(CHAR);
		if ( inputState->guessing==0 ) {

			
					char decoded_char = m_decode_char(c->getText());
					val = make_value_copy(decoded_char);
				

		}
		break;
	}
	case STRING:
	{
		s = LT(1);
		match(STRING);
		if ( inputState->guessing==0 ) {

			
					string decoded_string = m_decode_string(s->getText());
					val = make_value_copy(decoded_string);
				

		}
		break;
	}
	default:
	{
		throw antlr::NoViableAltException(LT(1), getFilename());
	}
	}
	return val;
}

Handle<Value>  xParamParser::compound_value() {

	Handle<Value> val;

	antlr::RefToken  tn = antlr::nullToken;

	
	ValueList args;
	

	
	try {      // for error handling
		tn = LT(1);
		match(TYPENAME);
		match(10);
		args=list_of_values();
		match(11);
		if ( inputState->guessing==0 ) {

			val=m_create_value(tn->getText(),args);

		}
	}
	catch (Error& ex) {
		if (inputState->guessing==0) {

			
					throw Error("while parsing constructor for '" + tn->getText() + "':\n" + ex.what());
				

		} else {
			throw ex;
		}
	}
	catch (RecognitionException& ex) {
		if (inputState->guessing==0) {

			
					throw Error("syntax error while parsing constructor for '" + tn->getText() + "'");
				

		} else {
			throw ex;
		}
	}
	return val;
}

Handle<Value>  xParamParser::list_value() {

	Handle<Value> val;


	
	ValueList values;
	

	
	try {      // for error handling
		match(12);
		values=list_of_values();
		match(13);
		if ( inputState->guessing==0 ) {

			val = make_value_copy(values);

		}
	}
	catch (Error& ex) {
		if (inputState->guessing==0) {

			
					throw Error("while reading list:\n" + string(ex.what()));
				

		} else {
			throw ex;
		}
	}
	catch (RecognitionException& ex) {
		if (inputState->guessing==0) {

			
					throw Error("syntax error while reading list");
				

		} else {
			throw ex;
		}
	}
	return val;
}

void xParamParser::member_assignment(
	AssignmentListener& listener
) {
	antlr::RefToken  id = antlr::nullToken;

	
	Handle<Value> val;
	

	
	try {      // for error handling
		id = LT(1);
		match(ID);
		match(7);
		val=global_scope_value();
		if ( inputState->guessing==0 ) {

			listener.notify(id->getText(),val);

		}
	}
	catch (Error& ex) {
		if (inputState->guessing==0) {

			
					throw Error("while parsing assignment to parameter '" 
							+ id->getText() + "':\n" + ex.what());
				

		} else {
			throw ex;
		}
	}
	catch (RecognitionException& ex) {
		if (inputState->guessing==0) {

			
					throw Error("syntax error parsing assignment to parameter '"
							+ id->getText() + "'");
				

		} else {
			throw ex;
		}
	}
}

void xParamParser::flag_setting(
	AssignmentListener& listener
) {
	antlr::RefToken  flag = antlr::nullToken;
	
	flag = LT(1);
	match(FLAG);
	if ( inputState->guessing==0 ) {

		
				string str = flag->getText();
				bool flag_value = (str[0] == '+');
				string flag_name = str.substr(1,str.size()-1);
				listener.notify(flag_name,make_value_copy(flag_value));
			

	}
}

Handle<Value>  xParamParser::global_scope_value() {

	Handle<Value> val;


	
	ValueList values;
	

	
	values=non_empty_list_of_values();
	if ( inputState->guessing==0 ) {

		
				if (values.size() == 1) { // it's a single value
					val = values[0];
				}
				else { // it's a list of values
					val = make_value_copy(values);
				}
			

	}
	return val;
}

ValueList  xParamParser::non_empty_list_of_values() {

	ValueList list;


	
	Handle<Value> val;
	

	
	try {      // for error handling
		val=value();
		if ( inputState->guessing==0 ) {

			list.push_back(val);

		}
		{
		for (;;) {
			if ((LA(1)==14)) {
				match(14);
				val=value();
				if ( inputState->guessing==0 ) {

					list.push_back(val);

				}
			}
			else {
				goto _loop19;
			}
			
		}
		_loop19:;
		}
	}
	catch (Error& ex) {
		if (inputState->guessing==0) {

			
					if (!list.empty())
						throw Error("while parsing element " 
								+ m_int_to_string(list.size())
								+ " of value list:\n" + ex.what());
					else
						throw;
				

		} else {
			throw ex;
		}
	}
	catch (RecognitionException& ex) {
		if (inputState->guessing==0) {

			
					throw Error("syntax error while parsing element "
							+ m_int_to_string(list.size())
							+ " of value list:\n" + ex.what());
				

		} else {
			throw ex;
		}
	}
	return list;
}

ValueList  xParamParser::list_of_values() {

	ValueList list;

	
	switch ( LA(1)) {
	case 11:
	case 13:
	{
		break;
	}
	case ID:
	case URL:
	case TYPENAME:
	case 12:
	case TENTATIVE:
	case CHAR:
	case STRING:
	{
		list=non_empty_list_of_values();
		break;
	}
	default:
	{
		throw antlr::NoViableAltException(LT(1), getFilename());
	}
	}
	return list;
}

const char* xParamParser::_tokenNames[] = {
	"<0>",
	"EOF",
	"<2>",
	"NULL_TREE_LOOKAHEAD",
	"\";\"",
	"ID",
	"URL",
	"\"=\"",
	"FLAG",
	"TYPENAME",
	"\"(\"",
	"\")\"",
	"\"[\"",
	"\"]\"",
	"\",\"",
	"TENTATIVE",
	"CHAR",
	"STRING",
	0
};



ANTLR_END_NAMESPACE
