/*	Copyright (C) 2001,2002,2003 Ronnie Maor and Michael Brand
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the copyright holders give permission
 * for additional uses of the text contained in its release of XPARAM.
 *
 * The exception is that, if you link the XPARAM library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the XPARAM library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * If you write modifications of your own for XPARAM, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */

#pragma warning (disable: 4786)

#include <algorithm>
#include "../xparam/xp_param_imp.h"

using namespace std;

namespace xParam_internal {

bool not_in_string(const char ch, const string& s) {
  return find(s.begin(),s.end(),ch)==s.end();
}

class not_in_ws {
  public:
    bool operator()(const char ch) {
      string ws=" \n\t\v\r\f";
      return not_in_string(ch,ws);
    }
};

class not_in_namechar {
  public:
    bool operator()(const char ch) {
      string namechar=
        "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
      return not_in_string(ch,namechar);
    }
};

void ParamPartialImp::separate_name_and_description(const string& s)
{
  string::const_iterator name_begin,name_end,desc_begin,desc_end;

  name_begin=find_if(s.begin(),s.end(),not_in_ws());
  name_end=find_if(name_begin,s.end(),not_in_namechar());

  if (name_begin==name_end) {
    throw Error("Not a legal name-description sequence (no name found): "+s);
  }
  if (not_in_string(*name_begin,
      "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")) {
    throw Error("No legal name-first character in name can not be a digit: "+s);
  }

  m_name.assign(name_begin,name_end);

  desc_begin=find_if(name_end,s.end(),not_in_ws());
  if (desc_begin==s.end()) {
    m_description="";
    return;
  }

  if ((*desc_begin)!='!') {
    throw Error("\'!\' expected before description in "+s);
  }

  desc_begin=find_if(++desc_begin,s.end(),not_in_ws());
  if (desc_begin==s.end()) {
    m_description="";
    return;
  }

  desc_end=desc_begin;
  for(string::const_iterator temp=desc_begin;temp!=s.end();++temp) {
    if (not_in_ws()(*temp)) { desc_end=temp; ++desc_end; }
  }

  m_description.assign(desc_begin,desc_end);
}

}
