/*	Copyright (C) 2001,2002,2003 Ronnie Maor and Michael Brand
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the copyright holders give permission
 * for additional uses of the text contained in its release of XPARAM.
 *
 * The exception is that, if you link the XPARAM library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the XPARAM library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * If you write modifications of your own for XPARAM, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */

#pragma warning (disable: 4786)

#include "../xparam/xpv_scheduler.h"
#include "../xparam/xpv_regcom.h"

using namespace std;

#include <iostream>

namespace xParam_internal {

	void RegistrationScheduler::add_command(Handle<RegistrationCommand> command)
	{
		m_commands.push_back(command);
		check_dependencies();
	}

	bool RegistrationScheduler::commands_pending() const
	{
		return !m_commands.empty();
	}

	/////////////////////////////////////////////////////////////////////////////
	// This method goes over the stored commands and executes those that have
	// their dependencies fulfilled.
	//
	// We must remember that commands may call the add_command(), and the
	// check_dendencies() methods during their registration.
	/////////////////////////////////////////////////////////////////////////////
	void RegistrationScheduler::check_dependencies()
	{
		if (m_already_checking)
			return;
		m_already_checking = true;

		while(true) {
			// collect all commands that can register themselves
			CommandList non_blocked;
			CommandList::iterator i = m_commands.begin();
			while (i != m_commands.end()) {
				CommandList::iterator j = i++;
				if ((*j)->can_register()) {
					non_blocked.splice(non_blocked.begin(),m_commands,j,i);
				}
			}

			if (non_blocked.empty())
				break; // we've reached equilibrium

			// register all commands in non_blocked_commands
			for (i=non_blocked.begin(); i!=non_blocked.end(); ++i)
				(*i)->do_registration();
		}

		m_already_checking = false;
	}

} // namespace xParam_internal

