/*	Copyright (C) 2001,2002,2003 Ronnie Maor and Michael Brand
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the copyright holders give permission
 * for additional uses of the text contained in its release of XPARAM.
 *
 * The exception is that, if you link the XPARAM library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the XPARAM library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * If you write modifications of your own for XPARAM, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */

#pragma warning (disable: 4786)

#include "../xparam/xpv_value_imp.h"
#include "../xparam/xpv_registry.h"
#include "../src_include/xpv_conversions.h"
#include "../src_include/xp_sstream.h"

using namespace std;

namespace xParam_internal {

	const Type& ValuePartialImp::static_type() const {
		return type_registry().type(static_type_info());
	}
	
	const Type& ValuePartialImp::dynamic_type() const {
		return type_registry().type(dynamic_type_info());
	}
	
	void ValuePartialImp::output(ostream& os) const {
		dynamic_type().output(os,*this);
	}
	
	Handle<Value> ValuePartialImp::convert_to(const type_info& target) const 
	{
		Handle<Value> val = get_handle();
		vector<ConvPath> best_paths = find_best_paths(val,target);

		if (best_paths.size() != 1) 
		{
			string source_type_name = static_type().name();
			string target_type_name = type_registry().type(target).name();

			Oss base_oss;
			base_oss << "from " << source_type_name;
			base_oss << " ( ";
			output(base_oss);
			base_oss << " ) "	<< " to " << target_type_name;
			string msg = base_oss.str();

			if (best_paths.empty())
				throw Error ("no conversion path found " + msg);
			if (best_paths.size() > 1) {
				Oss oss2;
				oss2 << "More than one conversion path found " << msg << ":" << endl;

				vector<ConvPath>::const_iterator i;
				for (i=best_paths.begin(); i!=best_paths.end(); ++i) {
					const ConvPath& path = *i;
					oss2 << "\t" << source_type_name;

					ConvPath::const_iterator j;
					for (j=path.begin(); j!=path.end(); ++j)
						oss2 << " -> " << type_registry().type(**j).name();
					oss2 << endl;
				}
				throw Error(oss2.str());
			}
		}
		
		const ConvPath& path = best_paths.front();
		assert(path.empty() || (*path.back() == target));
		
		return convert_along_path(val,path);
	}

} // namespace xParam_internal

