//  Simple base class for use with the Boost smart pointer demonstration
//  program.
//
//  Copyright 2002 Jim Hyslop, all rights reserved
// 
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//
// This software has been tested with version 1.28.0 of the Boost
// library (released May 16, 2002). See www.boost.org to download Boost.

#include "base.h"
#include <iostream>

using namespace boost;
using namespace std;

class Base::BaseImpl
{
public:
    BaseImpl() { cout << "Creating BaseImpl class" << endl; }
    BaseImpl(const BaseImpl &) { cout << "Copying BaseImpl class" << endl; }
    BaseImpl & operator=(const BaseImpl &)
    {
        cout << "Assigning BaseImpl class" << endl; 
        return *this;
    }
    ~BaseImpl() { cout << "Destroying BaseImpl class" << endl; }
    
};

// Important note: this class does not implement copy-on-write semantics.
// 
Base::Base()
: impl_(new BaseImpl)
{
    cout << "Creating new Base class, impl_ use count is " 
        << impl_.use_count() << endl;
}

Base::Base(const Base &other)
:
impl_(other.impl_)
{
    cout << "Copying Base class, impl_ use count is " 
        << impl_.use_count() << endl;
}

Base & Base::operator=(const Base &other)
{
    cout << "Assigning Base class; use counts before assignment: other: "
        << other.showUseCount() << "; this: "
        << this->showUseCount() << endl;

    { // block scope to demonstrate counts
        shared_ptr<BaseImpl> tmp(other.impl_);
        cout << "Assigning Base class; use counts before swap: other: "
            << other.showUseCount() << "; this: "
            << this->showUseCount() << "; tmp: "
            << tmp.use_count() << endl;
        tmp.swap(impl_);
        cout << "Assigning Base class; use counts after swap:  other: "
            << other.showUseCount() << "; this: "
            << this->showUseCount() << "; tmp: "
            << tmp.use_count() << endl;
    }
    cout << "Assigning Base class; use counts after assignment: other: "
        << other.showUseCount() << "; this: "
        << this->showUseCount() << endl;
    return *this;
}

Base::~Base()
{
    cout << "Destroying Base class" << endl;
}
