//  Shared pointer demonstration program
//
//  Copyright 2002 Jim Hyslop, all rights reserved
// 
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//
// This software has been tested with version 1.28.0 of the Boost
// library (released May 16, 2002). See www.boost.org to download Boost.

#include <boost/shared_ptr.hpp>
#include <iostream>

using namespace boost;
using namespace std;

class Simple
{
};

void dumpSharedPointerInfo(shared_ptr<Simple> & first,
                           shared_ptr<Simple> & second)
{
    cout << "\nfirst: " << first.use_count() << " [0x" << first.get() << "]"
        << "; second: " << second.use_count() << " [0x" << second.get() << "]";
}

void dumpSharedPointerInfo(shared_ptr<Simple> & first,
                           shared_ptr<Simple> & second,
                           shared_ptr<Simple> & third)
{
    dumpSharedPointerInfo(first, second);
    cout << "; third: " << third.use_count() << " [0x" << third.get() << "]"
        << endl;
}

void demoSharedPointerCounts()
{
    cout << "Simple shared_ptr counting demonstration\n"
            "----------------------------------------\n"
            "\tOutput format: use_count [address]\n";
    shared_ptr<Simple> firstPointer(new Simple);
    shared_ptr<Simple> secondPointer(new Simple);
    {
        shared_ptr<Simple> thirdPointer(new Simple);

        cout << "Three unique shared_ptrs";
        dumpSharedPointerInfo(firstPointer, secondPointer, thirdPointer);

        secondPointer = firstPointer;

        cout << "\nAfter assigning secondPointer = firstPointer";
        dumpSharedPointerInfo(firstPointer, secondPointer, thirdPointer);

        firstPointer.reset();
        cout << "\nAfter resetting first pointer";
        dumpSharedPointerInfo(firstPointer, secondPointer, thirdPointer);

        thirdPointer = secondPointer;
        cout << "\nAfter assigning thirdPointer = secondPointer";
        dumpSharedPointerInfo(firstPointer, secondPointer, thirdPointer);
    }
    cout << "\nAfter third goes out of scope";
    dumpSharedPointerInfo(firstPointer, secondPointer);

    firstPointer = secondPointer;
    cout << "\n\nAfter resurrecting first pointer";
    dumpSharedPointerInfo(firstPointer, secondPointer);

    firstPointer.reset();
    secondPointer.reset();
    cout << "\n\nAfter resetting both pointers";
    dumpSharedPointerInfo(firstPointer, secondPointer);
    cout << "\nNote that the NULL pointers are not shared\n";

    secondPointer=firstPointer;
    cout << "\nAfter assigning secondPointer=firstPointer";
    dumpSharedPointerInfo(firstPointer, secondPointer);
    cout << "\nNote that the NULL pointers are shared this time\n";
}
